#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include "def.h"

int main() {
	int sock, client_socket;
	int client_len;
	struct sockaddr_in server_address;
	struct sockaddr_in client_address;
	fd_set readfds, testfds;

	sock = socket(AF_INET, SOCK_STREAM, 0);

	server_address.sin_family = AF_INET;
	server_address.sin_addr.s_addr = htonl(INADDR_ANY);
	server_address.sin_port = htons(SPEECH_PORT);

	if (bind(sock, (struct sockaddr *)&server_address, sizeof(server_address)) == -1)
		die("speechd: bind() failed");

	/* Create a connection queue and initialize readfds to handle input from sock. */
	if (listen(sock, 5) == -1)
		die("speechd: listen() failed");

	FD_ZERO(&readfds);
	FD_SET(sock, &readfds);

	/* Now wait for clients and requests.
	 * Since we have passed a null pointer as the timeout parameter, no timeout will occur.
	 * The program will exit and report an error if select returns a value of less than 1.  */
	while(1) {
		char ch;
		int fd;
		int nread;

		testfds = readfds;

		printf("speech server waiting for clients...\n");

		if (select(FD_SETSIZE, &testfds, (fd_set *)0, (fd_set *)0, (struct timeval *) 0) < 1)
			die("speechd: select() failed");

		/* Once we know we've got activity,
		 * we find which descriptor it's on by checking each in turn using FD_ISSET. */

		for (fd = 0; fd < FD_SETSIZE; fd++) {
			if(FD_ISSET(fd,&testfds)) {

				if(fd == sock) {	/* activity is on sock (request for a new connection) */

					client_len = sizeof(client_address);
					client_socket = accept(sock, (struct sockaddr *)&client_address, &client_len);

					/* and we add the associated client_socket to the descriptor set. */
					FD_SET(client_socket, &readfds);
					printf("adding client on fd %d\n", client_socket);

				} else {	/* client activity */

					ioctl(fd, FIONREAD, &nread);

					if(nread == 0) {
						/* If close is received, the client has gone away and we remove it from the descriptor set. */
						close(fd);
						FD_CLR(fd, &readfds);
						printf("removing client on fd %d\n", fd);
					} else {
						/* Here we 'serve' the client. */
						read(fd, &ch, 1);
						sleep(5);
						printf("serving client on fd %d\n", fd);
						ch++;
						write(fd, &ch, 1);
					}
				}
			}
		}
	}
}
